@@ -2,10 +2,9 @@ from importlib import import_module
 import os
 import uuid
 import random
-from datetime import datetime
 import socket
 
-from sentry_sdk._compat import string_types, text_type, iteritems
+from sentry_sdk._compat import datetime_utcnow, string_types, text_type, iteritems
 from sentry_sdk.utils import (
     capture_internal_exceptions,
     current_stacktrace,
@@ -229,6 +228,14 @@ class _Client(object):
 
             self.session_flusher = SessionFlusher(capture_func=_capture_envelope)
 
+            self.metrics_aggregator = None  # type: Optional[MetricsAggregator]
+            if self.options.get("_experiments", {}).get("enable_metrics"):
+                from sentry_sdk.metrics import MetricsAggregator
+
+                self.metrics_aggregator = MetricsAggregator(
+                    capture_func=_capture_envelope
+                )
+
             max_request_body_size = ("always", "never", "small", "medium")
             if self.options["max_request_body_size"] not in max_request_body_size:
                 raise ValueError(
@@ -243,7 +250,7 @@ class _Client(object):
                 )
                 self.options["instrumenter"] = INSTRUMENTER.OTEL
                 _DEFAULT_INTEGRATIONS.append(
-                    "sentry_sdk.integrations.opentelemetry.OpenTelemetryIntegration",
+                    "sentry_sdk.integrations.opentelemetry.integration.OpenTelemetryIntegration",
                 )
 
             self.integrations = setup_integrations(
@@ -284,7 +291,7 @@ class _Client(object):
         # type: (...) -> Optional[Event]
 
         if event.get("timestamp") is None:
-            event["timestamp"] = datetime.utcnow()
+            event["timestamp"] = datetime_utcnow()
 
         if scope is not None:
             is_transaction = event.get("type") == "transaction"
@@ -560,7 +567,7 @@ class _Client(object):
         if should_use_envelope_endpoint:
             headers = {
                 "event_id": event_opt["event_id"],
-                "sent_at": format_timestamp(datetime.utcnow()),
+                "sent_at": format_timestamp(datetime_utcnow()),
             }
 
             if dynamic_sampling_context:
@@ -610,6 +617,8 @@ class _Client(object):
         if self.transport is not None:
             self.flush(timeout=timeout, callback=callback)
             self.session_flusher.kill()
+            if self.metrics_aggregator is not None:
+                self.metrics_aggregator.kill()
             if self.monitor:
                 self.monitor.kill()
             self.transport.kill()
@@ -632,6 +641,8 @@ class _Client(object):
             if timeout is None:
                 timeout = self.options["shutdown_timeout"]
             self.session_flusher.flush()
+            if self.metrics_aggregator is not None:
+                self.metrics_aggregator.flush()
             self.transport.flush(timeout=timeout, callback=callback)
 
     def __enter__(self):
