@@ -25,6 +25,7 @@ if TYPE_CHECKING:
         ProfilerMode,
         TracesSampler,
         TransactionProcessor,
+        MetricTags,
     )
 
     # Experiments are feature flags to enable and disable certain unstable SDK
@@ -34,12 +35,16 @@ if TYPE_CHECKING:
     Experiments = TypedDict(
         "Experiments",
         {
+            "attach_explain_plans": dict[str, Any],
             "max_spans": Optional[int],
             "record_sql_params": Optional[bool],
             # TODO: Remove these 2 profiling related experiments
             "profiles_sample_rate": Optional[float],
             "profiler_mode": Optional[ProfilerMode],
             "otel_powered_performance": Optional[bool],
+            "transport_zlib_compression_level": Optional[int],
+            "enable_metrics": Optional[bool],
+            "before_emit_metric": Optional[Callable[[str, MetricTags], bool]],
         },
         total=False,
     )
@@ -165,6 +170,13 @@ class OP:
     FUNCTION = "function"
     FUNCTION_AWS = "function.aws"
     FUNCTION_GCP = "function.gcp"
+    GRAPHQL_EXECUTE = "graphql.execute"
+    GRAPHQL_MUTATION = "graphql.mutation"
+    GRAPHQL_PARSE = "graphql.parse"
+    GRAPHQL_RESOLVE = "graphql.resolve"
+    GRAPHQL_SUBSCRIPTION = "graphql.subscription"
+    GRAPHQL_QUERY = "graphql.query"
+    GRAPHQL_VALIDATE = "graphql.validate"
     GRPC_CLIENT = "grpc.client"
     GRPC_SERVER = "grpc.server"
     HTTP_CLIENT = "http.client"
@@ -271,4 +283,4 @@ DEFAULT_OPTIONS = _get_default_options()
 del _get_default_options
 
 
-VERSION = "1.31.0"
+VERSION = "1.32.0"
